#############################
### Randomization schemes ###
#############################

# (ii)-(a) We expect to randomize 100 patients
# Calculate the probability of obtaining an equal sample size
# in a randomization of 100 patients using the binomial distribution
# More information on the binomial distribution can be found with '?dbinom'
N = 100
dbinom(x = N/2, size = N, p = 0.5)

# (ii)-(b)
# Simulate a simple randomization example with 100 patients into two treatment groups
p = 0.5
N = 100
data = data.frame(id = paste("id", 1:N, sep = ""), trt = NA)
head(data)

# Set a seed for reproducibility of randomization results
set.seed(125)

# Simulate 100 uniform random variables
u = runif(N)
hist(u)

# Assign treatments based on random variables
data$trt[u < 0.5] = "A"
data$trt[u > 0.5] = "B"
data[1:10,]

# Display the distribution of patients in each treatment group
table(data$trt)
table(data$trt) / N

# (ii)-(c)
# Repeat the simple randomization process 1000 times and record results
K = 1000
set.seed(2342)

results = data.frame(replication = 1:K, NumberA = NA, NumberB = NA)

for (i in 1:K) {
  data = data.frame(id = paste("id", 1:N, sep = ""), trt = NA)
  u = runif(N)
  
  # Assign treatments based on random variables
  data$trt[u < 0.5] = "A"
  data$trt[u > 0.5] = "B"
  
  # Record the number of patients in each treatment group for each replication
  results[i, "NumberA"] = table(data$trt)["A"]
  results[i, "NumberB"] = table(data$trt)["B"]
}
head(results)

# Calculate the proportion of replications with an imbalance greater than 60/40
table(results[, 2] > 60 | results[, 2] < 40) / K

# Display means and variances of the number of patients in each treatment group
# Note the tendency toward equal numbers in each treatment group over multiple replications
apply(results[, 2:3], 2, mean)
apply(results[, 2:3], 2, var)

# Set the working directory for saving graphs
setwd("C:/Clinical trials/R labs/lab3/graphs")

# Note: 'Na' appears to converge to 50, indicating a balanced allocation
pdf("simple_rand.pdf", height = 5.5, width = 5.5)
plot(cumsum(results[, 2]) / 1:K, type = "l", xlab = "Replication index",
     ylab = "Mean number of patients on treatment A")
dev.off()

# (iii) Randomization in blocks
# (a) Block size 4 with two treatments A and B
# Calculate the number of permutations possible within the block
choose(4, 2)

# Visualize the permutations using the Deducer package
library(Deducer)
t(perm(c("A", "A", "B", "B")))

# (c) Randomize 100 subjects using random block sizes
library(blockrand)
# For information on the blockrand function
?blockrand

# Set a seed for reproducibility of randomization results
set.seed(1322)

# Generate random block sizes and allocate 100 subjects into treatments A and B
block_rand = blockrand(n = 100, num.levels = 2, levels = c("A", "B"), 
                       block.sizes = 1:4)
# Display a subset of the randomization with block ids 1 and 2
block_rand[block_rand$block.id %in% 1:2,]

# Total number of randomizations performed
nrow(block_rand)

# Display the count of patients in each treatment group
table(block_rand$treatment[1:100])


# (iv) Blocking within strata
# (b) Stratification by sex
set.seed(2435)
# Generate randomized blocks for males and females separately
male = blockrand(n = 100, id.prefix = 'M', block.prefix = 'M', stratum = 'Male')
female = blockrand(n = 100, id.prefix = 'F', block.prefix = 'F', stratum = 'Female')

# Restrict the number of participants to 55 men and 45 women at the study's conclusion
male = male[1:55,]
female = female[1:45,]

# Combine male and female datasets into one
mystudy = rbind(male, female)

# Count of subjects in the two treatment groups
table(mystudy$treatment)

# Create a two-way contingency table
x = table(mystudy$treatment, mystudy$stratum)
x

# Conduct a chi-square test for independence
# Simulate p-value 10,000 times for accurate results
chisq.test(x, simulate.p.value = TRUE, B = 10000)


# Urn schemes
# Install or update R packages if needed (commented out for clarity)
# library(installr)
# updateR()

# Set a seed for reproducibility
set.seed(12)

# Load the 'randomizeR' package for urn-based randomization
library(randomizeR)

# Create an urn with specified parameters and generate a sequence
x = udPar(N = 2500, ini = 1, add = 1, groups = LETTERS[1:2])
y = genSeq(x, r = 1, seed = 12)
y

# Display a subset of the generated sequence
c(getRandList(y))[1:10]

# Count occurrences of elements in the urn
table(c(getRandList(y)))

# Function to simulate an urn model for randomization
urnModel = function(n) {
  na = 1
  nb = 1
  
  # Initialize vectors to store probabilities and treatment assignments
  prob = rep(NA, n)
  trt = rep(NA, n)
  u = runif(n)
  
  for (i in 1:n) {
    # Calculate the probability of getting treatment A
    prob[i] = na / (na + nb)
    
    # Assign treatments based on generated probabilities
    if (u[i] < prob[i]) {
      trt[i] = "A"
      nb = nb + 1
    } else {
      trt[i] = "B"
      na = na + 1
    }
  }
  
  return(data.frame(treatment = trt, probA = prob, na = na, nb = nb))
}


# Simulate random draws from the urn model
draws = urnModel(n = 2500)
head(draws)

# Count occurrences of treatments A and B
table(draws$treatment)

# Set the working directory for saving graphs
setwd("C:/Clinical trials/R labs/lab3/graphs")

# Generate a plot illustrating the probabilities of getting treatment A
pdf("urn.pdf", height = 5.5, width = 5.5)
plot(1:2500, draws$probA, ylab = "Probability", xlab = "Number of subjects",
     main = "Probability of getting A based on the urn model", type = "l")
abline(h = 0.5, col = "red") # Add a line at 0.5 for reference
dev.off()



################
### Optional ###
################

# Set the seed for reproducibility
set.seed(5)

# Define the number of subjects and iterations
N = 14
it = 200

# Results dataframe to store simulation outcomes
results = data.frame(SR = rep(NA, it), BR = NA, urn = NA)

for (i in 1:it) {
  ########################
  # Simple randomization #
  ########################
  
  # Generate a dataframe for simple randomization
  data = data.frame(id = paste("id", 1:N, sep = ""), trt = NA)
  
  # Simulate random values 'u' to decide treatment assignment
  u = runif(N)
  
  # Assign patients to treatment arms based on 'u'
  data$trt[u < 0.5] = "A"
  data$trt[u > 0.5] = "B"
  
  # Calculate the proportion of patients receiving treatment A
  results$SR[i] = table(data$trt)[1] / N
  
  ###########################
  # Randomization in blocks #
  ###########################
  
  # Generate randomization in blocks for 'N' subjects with block sizes 1 to 4
  block_rand = blockrand(n = N, num.levels = 2, levels = c("A", "B"), block.sizes = 1:4)[1:N,]
  
  # Calculate the proportion of patients receiving treatment A in block randomization
  results$BR[i] = table(block_rand$treatment)[1] / N
  
  ##############
  # Urn scheme #
  ##############
  
  # Simulate an urn model for randomization
  data = urnModel(N)
  
  # Calculate the proportion of patients receiving treatment A in urn-based randomization
  results$urn[i] = table(data$treatment)[1] / N
}

# Expected long-term proportions for each randomization scheme
apply(results, 2, mean)

# Standard deviation for each randomization scheme
apply(results, 2, sd)

# Visual representation of the evolution of assignment proportions over iterations
pdf("sim_results.pdf", height = 5.5, width = 5.5)
plot(cumsum(results$SR) / 1:it, type = "l", lty = 1, col = "black",
     ylim = c(0.40, 0.60), ylab = "Proportion of assigning A", xlab = "Draws")
lines(cumsum(results$BR) / 1:it, lty = 2, col = "red")
lines(cumsum(results$urn) / 1:it, lty = 3, col = "blue")

# Legend for the plot
legend("topright", lty = 1:3, col = c("black", "red", "blue"),
       legend = c("Simple randomization", "Randomization in blocks", "Urn scheme"), bty = "n")
dev.off()