################################################
### Sequential monitoring of clinical trials ###
################################################


# Load necessary library for clinical trial monitoring
library(gsDesign)

#####################################
# Pocock error spending boundaries
#####################################

# Create design
x1 = gsDesign(k = 5, test.type = 2, alpha = 0.025, sfu = "Pocock")

# Print boundary summary
gsBoundSummary(x1)

# Display plot for Pocock boundaries
plot(x1, main = "")

# Extract Z-values and p-values for Pocock boundaries
pocock_summary = gsBoundSummary(x1)
bind(Z = pocock_summary$Eff[pocock_summary$Val == "Z"],
     P = 2 * pocock_summary$Eff[pocock_summary$Val == "p (1-sided)"])

# Output specific information from the Pocock boundaries summary
pocock_output <- capture.output(gsBoundSummary(x1))
cat(pocock_output[23], "\n")



###############################
# O'Brien-Fleming boundaries
###############################

# Create new design
x2 = gsDesign(k = 5, test.type = 2, alpha = 0.025, sfu = "OF")

# Print boundary summary
gsBoundSummary(x2)

# Extract Z-values and p-values for O'Brien-Fleming boundaries
of_summary <- gsBoundSummary(x2)
cbind(Z = of_summary$Eff[of_summary$Val == "Z"],
      P = 2 * of_summary$Eff[of_summary$Val == "p (1-sided)"])

# Output specific information from the O'Brien-Fleming boundaries summary
of_output <- capture.output(gsBoundSummary(x2))
cat(of_output[23], "\n")

# Display plot for O'Brien-Fleming boundaries
par(mar = c(4, 4, 1, 1))
plot(x2, main = "") 


#################################  
# Linear spending function
#################################

# Create design with a linear spending function
t <- c(0.2, 0.4, 0.6, 0.8, 1.0)
x3 <- gsDesign(k = 5, test.type = 2, beta = 0.2, sfu = sfLinear, sfupar = c(t, t))
x3

# Plot boundaries
# Pictorial representation of the design featuring a linear spending function
par(mar = c(4, 4, 1, 1))
plot(x3, main = "") +
  theme(legend.position = "none") +
  theme(axis.text = element_text(size = 10)) +
  theme(axis.title = element_text(size = 15, face = "bold"))


# Plot different spending functions
t<-seq(0, 1,.05)

p08=0.05*t^0.8 # Pocock
p1=0.05*t # Linear
p3=0.05*t^3.0 # O-F

plot(t, p08, lty=2, type="l", ylab="Alpha spent", xlab="Trial fraction", main="")
lines(t, p1, lty=1, lwd=2)
lines(t, p3, lty=3, lwd=2)

legend(0,.05, legend = c(expression(paste(rho, " = ", 0.8, " (Pocock)")),
                         expression(paste(rho, " = ", 1, " (linear)")),
                         expression(paste(rho, " = ", 3, " (O-F)"))), 
       lty=c(2,1,3), lwd=c(2,2,2), title="Power spending functions")


# Plot of HSD error spending functions ??(t)=??*(1-exp(-??*t))/(1-exp(-??))
t <- seq(0, 1, 0.05)
p08 = 0.05 * t^0.8  # Pocock
p1 = 0.05 * t  # Linear
p3 = 0.05 * t^3.0  # O-F

plot(t, p08, lty = 2, type = "l", ylab = "Alpha spent", xlab = "Trial fraction", main = "")
lines(t, p1, lty = 1, lwd = 2)
lines(t, p3, lty = 3, lwd = 2)

legend(
  0, .05, legend = c(
    expression(paste(rho, " = ", 0.8, " (Pocock)")),
    expression(paste(rho, " = ", 1, " (linear)")),
    expression(paste(rho, " = ", 3, " (O-F)"))
  ),
  lty = c(2, 1, 3), lwd = c(2, 2, 2), title = "Power spending functions"
)


# Inserting an additional interim analysis
# Create interim analyses with varying parameters and display them
t<-c(0.2, 0.5, 1.0)
x4<-gsDesign(k=3, test.type=1, sfu=sfLinear, sfupar=c(t,t), timing=t)
x4

# adding an interim look at t = 0.75
t<-c(0.2, 0.5, 0.75, 1.0)
x5<-gsDesign(k=4, test.type=1, sfu=sfLinear, alpha=0.025, beta=0.10, sfupar=c(t,t), timing=t)
x5

# Evaluate sample size inflation for HSD spending function parameter variations
x5 <- array(dim = 11)
for (i in 1:11) {
  x5[i] <- gsDesign(k = 5, test.type = 2, alpha = 0.025, sfu = sfHSD, sfupar = -i + 1)$en[1]
}

par(mar = c(4, 4, 1, 1))
plot(seq(0, -10, -1), x5, type = "b", ylab = "Sample size inflation",
     xlab = expression(paste(gamma, " parameter in the HSD spending function")))
