################################################
### Sequential monitoring of clinical trials ###
################################################

# Load the necessary library for clinical trial monitoring
library(gsDesign)

#############################################################
# Fixed sample size design calculations for clinical trials
#############################################################

# Set parameters
HR = 0.75
theta = (qnorm(0.975) + qnorm(0.80))

# Calculate the number of events without monitoring
D = 4 * theta^2 / log(HR)^2
D

################################
### Translate to sample size ###
################################

### Hazard rate for control    
m0 <- 9
lambda0 <- -log(0.5)/m0

# Hazard rate for experimental treatment
m1 <- 12
lambda1 <- -log(0.5)/m1

# Sample size calculations for fixed-sample designs
nSurv(alpha = 0.05, sided = 2, beta = 0.20,
      lambdaC = lambda0, hr = lambda1 / lambda0, hr0 = 1,
      gamma = 30, T = 36, ratio = 1)

nSurv(alpha = 0.05, sided = 2, beta = 0.20,
      lambdaC = lambda0, hr = lambda1 / lambda0, hr0 = 1,
      gamma = 40, T = 36, ratio = 1)

# Add interim analyses for monitoring
gsDesign(k = 3, test.type = 1, alpha = 0.025, beta = 0.20, n.fix = 380,
         timing = c(0.3, 0.6, 1), sfu = "OF")

#################################
# Design with gsSurv function
#################################

# Design survival trial 
x3 = gsSurv(k = 3, test.type = 2, alpha = 0.05, sided = 2,
            lambdaC = lambda0, hr = lambda1 / lambda0, hr0 = 1,
            beta = 0.20, sfu = "OF", gamma = 40, T = 36,
            ratio = 1, timing = c(0.3, 0.6, 1.0))
x3

#########################################
### Recalibration of survival studies ###
#########################################

# Define fractions and spending functions
t <- seq(0,1,.1)

# Original spending function
apt <- 0.05 * log(1 + (exp(1) - 1) * t)
cbind(t, apt)

par(mar = c(4, 4, 1, 1))
# Plot original and recalibrated spending functions
plot(t, apt,
     xlab = "Trial fraction", ylab = "Alpha spent",
     type = "l", col = "blue")
abline(v = 0.2, lty = 2)

# Recalibrated spending function
apt1 <- apt[2] + ((0.05 - apt[2]) / (0.05 - apt[3])) * (apt - apt[3])
lines(t[t >= 0.2], apt1[t >= 0.2], col = "red")

# Study monitoring using calendar time with different design parameters
gsDesign(k = 6, test.type = 2, alpha = 0.025, beta = 0.1, sfu = sfLDPocock)


#######################
### Futility bounds ###
#######################

# (v) Set values for futility bounds
m0 = 8
HR = 2/3
lambda0 = - log(0.5)/m0

# Calculate parameters for futilit
m1 = m0/HR
lambda1 = - log(0.5)/m1

# (b) Sequential monitoring of survival study using non-binding futility bounds

x = gsSurv(k = 5, test.type = 4, alpha = 0.05,sided = 2,
           lambdaC = lambda0, hr = lambda1/lambda0, hr0 = 1,
           beta = 0.20, sfu = sfHSD,sfupar = -4 ,sfl = sfHSD, 
           gamma = 20, R = 15 , ratio = 1)
x

# Visualize efficacy and non-binding futility boundaries
plot(x, main = "Efficacy & non-binding futility boundaries",
     ylim = c(-3.5, 3.5), plottype = 1, xlab = "Number of events")

# Sequential monitoring of survival study using binding futility bounds
x = gsSurv(k = 5, test.type = 3, alpha = 0.05, sided = 2,
           lambdaC = lambda0, hr = lambda1 / lambda0, hr0 = 1,
           beta = 0.20, sfu = sfHSD, sfupar = -4, sfl = sfHSD,
           gamma = 20, R = 15, ratio = 1)
x

# Visualize efficacy and binding futility hazard ratios
plot(x, main = "Efficacy & non-binding futility hazard ratios",
     plottype = "HR", xlab = "Number of events")